import gc
import os
import time
import medhalib.Utils.LogModules as LogModule

# import mock while in development/debug mode
if ("DEBUGMODE" in os.environ):
  import Mock.GPIO as GPIO
else:
  import RPi.GPIO as GPIO

# initialize logger
log = LogModule.init_logger()

class UltraSonicSensor:

  """
  UltraSonicSensor class implements interface for ultrasonic sensor module.
  """

  def __init__(self, trig_pin, echo_pin):
    # pin values in GPIO.BCM numbers
    GPIO.setmode(GPIO.BCM)

    self.TRIGGER_PIN = trig_pin
    self.ECHO_PIN = echo_pin

    GPIO.setup(self.TRIGGER_PIN, GPIO.OUT)
    GPIO.setup(self.ECHO_PIN, GPIO.IN)

  # method that gets distance value from sensor
  def get_distance(self):
    log.debug(LogModule.METHOD_ENTRY)

    GPIO.output(self.TRIGGER_PIN, True)
    time.sleep(0.00001)
    GPIO.output(self.TRIGGER_PIN, False)
    start_time = 0
    stop_time = 0
    timeout = 20
    timeoutstart = time.time()

    while GPIO.input(self.ECHO_PIN) == 0 and (time.time() - timeoutstart) <= timeout:
      start_time = time.time()

    while GPIO.input(self.ECHO_PIN) == 1 and (time.time() - timeoutstart) <= timeout:
      stop_time = time.time()

    distance = ((stop_time - start_time) * 34300) / 2

    del start_time
    del stop_time
    log.debug(LogModule.METHOD_EXIT)

    return distance

  # method to cleanup sensor resources
  def cleanup(self):
    log.debug("cleaning GPIO pins from Ultrasonic sensor module")
    GPIO.cleanup()



import logging

LOGGER_NAME = "MEDHABOARD"
METHOD_ENTRY = "Entered method"
METHOD_EXIT = "Exit method"

def init_logger():
    logging.getLogger("paramiko").setLevel(logging.DEBUG)
    # init log module
    logging.basicConfig(
        format='%(asctime)-15s - %(levelname)8s - %(module)10s [%(filename)s:%(lineno)d] - %(message)s',
        level=logging.DEBUG,
        datefmt='%m/%d/%Y %I:%M:%S.%p',
        handlers=[
            # logging.FileHandler("poc1.log"),
            logging.StreamHandler()
        ]
    )
    return logging.getLogger(LOGGER_NAME)



import gc
import os
import time
import warnings
import medhalib.Utils.LogModules as LogModule

# import mock while in development/debug mode
if ("DEBUGMODE" in os.environ):
  import Mock.GPIO as GPIO
else:
  import RPi.GPIO as GPIO

# initialize logger
log = LogModule.init_logger()


class Servo180:

  """
  Servo180 class contains functionality to set 180 degree servo to an angle.
  """

  def __init__(self, servo_pin):
    # pin values in GPIO.BCM numbers
    GPIO.setmode(GPIO.BCM)

    self.SERVO_PIN = servo_pin

    GPIO.setup(self.SERVO_PIN, GPIO.OUT)

    self.SERVO_PWM = GPIO.PWM(self.SERVO_PIN, 50)         # GPIO servo_pin for PWM with 50Hz
    self.SERVO_PWM.start(0)                               # Initialization


  # methog to convert angle to cycles
  def get_cycle_from_angle(self, angle):
    duty = angle / 18 + 2
    log.debug("Ange: {angle} -> Duty: {duty}".format(angle=angle, duty=duty))
    return duty


  def set_servo_angle(self, angle):
    # check that angle must be within 0-180
    if (angle < 0 or angle > 180):
      errmsg = "angle value must be between 0 to 180. supplied: {angle}".format(angle=angle)
      log.warning(errmsg)
      warnings.warn(errmsg)
    else:
      log.debug(LogModule.METHOD_ENTRY)

      # set SERVO_PIN to output mode
      GPIO.output(self.SERVO_PIN, True)

      self.SERVO_PWM.ChangeDutyCycle(self.get_cycle_from_angle(angle))
      time.sleep(1)

      # turn off SERVO_PIN output mpde
      GPIO.output(self.SERVO_PIN, False)
      self.SERVO_PWM.ChangeDutyCycle(0)

      log.debug(LogModule.METHOD_EXIT)



import site
import os.path

# read version details from version.txt
def read_version():
    version_file = """Version: 2.0.3 [Date: 2022-04-17 19:09:03.258586]"""
    return version_file

__version__=read_version()


import os
import shutil
import time
import datetime
from setuptools import setup, find_packages

# build directory
distdir = "dist"

# cleanup if exist before build
if (os.path.exists(distdir)):
  print("cleaning {}".format(distdir))
  shutil.rmtree(distdir)

# timestampted version suffix
build_datetime = datetime.datetime.now()
version = '2.0.3'
print("version: {}".format(version))

# write build details to version.txt
with open(os.path.join("src", "medhalib", "__init__plh.py"), "r") as vflr:
  filedt = vflr.read()
  details = str("""Version: {version} [Date: {date}]""").format(version=version, date=build_datetime)
  filedt = filedt.replace("$VERSION_INFO_PLACEHOLDER", details)

  with open(os.path.join("src", "medhalib", "__init__.py"), "w") as vflw:
    vflw.write(filedt)

setup(
  name = 'medhalib',
  version=version,
  license='MIT',
  author="ConundrumArchives",
  author_email="team@conundrumarchives.space",
  packages=find_packages('src', exclude="tests"),
  package_dir={'': 'src'},
  data_files=[
    # ('', ["src/medhalib/version.txt"])
  ],
  include_package_data=True,
  tests_require = [
    "pytest"
  ],
  test_suite="tests",
  url='https://conundrumarchives.org/medha',
  keywords='Conundrum Archives Medha'
)



import os
import base64


# import mock while in development/debug mode
if ("DEBUGMODE" in os.environ):
  import sys
  import fake_rpi
  try:
    from picamera import PiCamera
  except:
    sys.modules['picamera'] = fake_rpi.picamera
else:
  from picamera import PiCamera

class CameraModule:

  """
  CameraModule class interfaces camera component and includes:
  - functionality to capture with specifications
  - get base64 format of the image
  """

  def __init__(self, resolution="MAX", rotate=0, capturefile="capture.jpg"):
    self.camera = PiCamera()

    # check for resolution
    if (resolution == "MAX"):
      self.camera.resolution = self.camera.MAX_RESOLUTION
    elif (type(resolution) is tuple):
      self.camera.resolution = resolution
    else:
      raise ValueError("invalid resolution value supplied. expecting MAX or tuple of (width, height) value in pixels")

    # check for rotate value
    if (type(rotate) is not int):
      raise ValueError("value of rotate must be int")
    elif(rotate < 0 or rotate > 360):
      raise ValueError("value of rotate must be between 0 and 360")
    else:
      self.camera.rotation = rotate

    # check for capturefile
    if((type(capturefile) is not str)):
      raise ValueError("check the value of capturefile")
    elif(not capturefile.endswith(".jpg") and not capturefile.endswith(".png")):
      raise ValueError("capturefile extension must be either .png or .jpg")
    else:
      self.capturefile = capturefile

  # method to capture from camera
  def capture(self, capturefile=None):
    if(capturefile is None):
      capturefile = self.capturefile
    
    self.camera.capture(capturefile)

  # method to get base64 form of the captured image
  def get_base64_image(self):
    return str(base64.b64encode(open(self.capturefile,"rb").read()).decode("ascii"))

  # method to close and cleanup camera resource
  def cleanup(self):
    self.camera.close()